@@ -7,7 +7,6 @@ import redis
 import json
 import time
 import socket
-import logging
 import subprocess
 from tabulate import tabulate
 from pathlib import Path
@@ -24,10 +23,10 @@ def get_time():
     return str(datetime.datetime.now()).split('.')[0]
 
 def get_host():
-    return socket.gethostname().split('.', 1)[0]
+    return socket.gethostname().split('.', 1)[0] + " " + str(os.getpid())
 
 def table_list(data, headers = None):
-    return tabulate([json.loads(d.decode("utf-8")) for d in data], headers = headers)
+    return tabulate(data, headers = headers)
 
 def _log(info, color):
     csi = '\033['
@@ -54,12 +53,14 @@ class CAM(object):
         if not os.path.exists(CONFIG_FILE):
             open(CONFIG_FILE, "w").write(DEFAULT_CONF)
         self._conf = yaml.load(open(CONFIG_FILE).read(), yaml.FullLoader) 
-        self._redis = redis.StrictRedis(host=self._conf["server"], port=self._conf["port"], password=self._conf["password"], db=0) 
+        self._redis = redis.StrictRedis(host=self._conf["server"], port=self._conf["port"], password=self._conf["password"], db=0)
 
     def __del__(self):
         if hasattr(self, "running_job_id"):
             self._remove_by_tid("running", self.running_job_id)
             self.p.kill()
+        if hasattr(self, "worker_id"):
+            self._redis.hdel("workers", self.worker_id)
 
     def _remove_by_tid(self, part, tid):
         part_str = self._redis.lrange(part, 0, -1)
@@ -102,7 +103,7 @@ class CAM(object):
         port = self._conf["port"] if port is None else port
         os.system("redis-server --port {0} --requirepass {1}".format(port, self._conf["password"]))
 
-    def worker(self, cond = ""):
+    def worker(self, cond = "", cmdprefix = "", cmdsuffix = ""):
         """
          Start the worker. 
         <br>`cam worker "some start condition"`
@@ -112,14 +113,26 @@ class CAM(object):
         <br>Slurm node count\t: "bash('squeue').count('ltl-gpu')<4"
         """
         log_info("Worker started. Listening to {0} ...".format(self._conf['server']))
+        worker_start_time = get_time()
+        self.worker_id = get_host()
+        os.system("tmux rename-window cam%d"%os.getpid())
+        self._redis.hset("workers", self.worker_id, json.dumps([worker_start_time, "Wait Task"]))
         while True:
             cnt = self._redis.llen("pending")
-            if cnt > 0 and self._condition_parse(cond):
+            if cnt <= 0:
+                self._redis.hset("workers", self.worker_id, json.dumps([worker_start_time, "Wait Task", cond, cmdprefix, cmdsuffix]))
+            elif not self._condition_parse(cond):
+                self._redis.hset("workers", self.worker_id, json.dumps([worker_start_time, "Wait Resource", cond, cmdprefix, cmdsuffix]))
+            else:
                 row_str = self._redis.rpop("pending")
+                if row_str is None:
+                    continue
                 self.running_job_id, ptime, cmd, status = json.loads(row_str.decode("utf-8"))
+                cmd = "".join([cmdprefix, cmd, cmdsuffix])
                 self._redis.lpush("running", json.dumps([self.running_job_id, get_time(), cmd, get_host()]))
                 log_info("Running command: ", cmd)
                 self.p = Popen(cmd, shell=True)
+                self._redis.hset("workers", self.worker_id, json.dumps([worker_start_time, "Running %d"%self.running_job_id, cond, cmdprefix, cmdsuffix]))
                 while True:
                     try:
                         if self._get_by_tid("running", self.running_job_id)[-1] == "KILLED":
@@ -151,14 +164,33 @@ class CAM(object):
             self._redis.rpush("pending", json.dumps([cnt, get_time(), cmd, "Pending"]))
         self._redis.set('jobid', cnt + 1)
 
-    def ls(self):
+    def ls(self, type = None):
         """
         Show the status of all tasks.
         """
-        log_info("Server: ", self._conf['server'])
-        pending = self._redis.lrange("pending", 0, -1)
-        running = self._redis.lrange("running", 0, -1)
-        print(table_list(pending + running, headers = ["ID", "Time", "Command", "Host"]))
+        now = datetime.datetime.now()
+        if type is None or type == "task":
+            log_info("Server: ", self._conf['server'])
+            pending = self._redis.lrange("pending", 0, -1)
+            running = self._redis.lrange("running", 0, -1)
+            res = pending + running
+            nres = []
+            for i in range(len(res)):
+                data = json.loads(res[i].decode("utf-8"))
+                st = datetime.datetime.fromisoformat(data[1])
+                data[1] = str(now - st).split('.')[0]
+                nres.append(data)
+            print(table_list(nres, headers = ["ID", "Time", "Command", "Host/PID"]))
+            print()
+        if type is None or type == "worker":
+            workers = self._redis.hgetall("workers")
+            info = []
+            for w in workers:
+                dt = json.loads(workers[w].decode("utf-8"))
+                st = datetime.datetime.fromisoformat(dt[0])
+                lst = [w, str(now - st).split('.')[0]] + dt[1:]
+                info.append(lst)    
+            print(table_list(info, headers = ["Worker/PID", "Up Time", "Status", "cond", "prefix", "suffix"]))
 
     def config(self):
         """
